// Drew Waranis

#include <stdio.h>
#include <iostream>
#include <vector>
#include <memory>

#include "allocore/io/al_App.hpp"
#include "allocore/graphics/al_Font.hpp"

#include "Gamma/Oscillator.h"

using namespace al;

// Data formatted by Ruby script to be easy to paste in.
int data[84] = {46., 41., 63., 44., 48., 62., 64., 82., 93., 115., 97., 76., 48., 75., 105., 100., 116., 81., 94., 88., 83., 81., 88., 83., 90., 83., 92., 111., 86., 88., 89., 86., 85., 70., 65., 68., 108., 88., 96., 108., 114., 131., 121., 129., 99., 173., 175., 190., 202., 164., 193., 182., 167., 159., 148., 115., 104., 123., 104., 103., 103., 91., 114., 99., 83., 92., 76., 84., 66., 87., 74., 84., 84., 71., 83., 59., 49., 63., 79., 51., 60., 79., 58., 64.};

// Ratio of window width to height.
double w = 3.2;

class MyApp : public App{
public:

	MyApp() {
		initWindow(Window::Dim(1920,600), "Pearl Jam's Popularity at SPL from 2006 to 2012", 40, Window::DEFAULT_BUF);
		window().remove(navControl());
	}

	// Graphics callbacks
	virtual void onAnimate(double dt){}
	virtual void onDraw(Graphics& gl, const Viewpoint& v){

    gl.clearColor(.0, .0, .0, .0);
		gl.clear(gl.COLOR_BUFFER_BIT);

		gl.matrixMode(gl.PROJECTION);
		//gl.pushMatrix();
		gl.modelView(Matrix4d::identity());
		// Scale dimensions to just larger than the outermost point
		gl.projection(Matrix4d::ortho(-.01,w+.01,0.0,1.01,0.0,1.0));

		gl.matrixMode(gl.MODELVIEW);
		gl.modelView(Matrix4d::identity());

    // Vertices defining each bar.
    double x1, x2, y, y2;

    // Save 1/8 of the screen for a key.
    double off = w/8.;

    // Largest element in data.
    // Currently picked by hand.
    double max = 202.;

    // Hard coded to 7 years * 12 months.
	  for (int i = 0; i < 7*12; ++i){
	  	// Only calculate these once per loop.
	    x1 = off + i*(w-off)/(7.*12.);
	    x2 = off + (i+1)*(w-off)/(7.*12.);
	     // Normalize data to window OpenGL coordinate system.
	    y = data[i] / max;

      // Double data point for last month.
      if (i < 7*12 - 1){
      	y2 = data[i+1] / max;
      }
      else {
        y2 = data[i] / max;
      }

      // Outlines bars on left and top
      // for a psuedo 3D effect.
	    gl.color(.5,.5,.5);
	    gl.lineWidth(1.5);
	    gl.begin(gl.LINE_STRIP);
	      gl.vertex(x1, 0.);
	      gl.vertex(x1, y);
	      gl.vertex(x2, y2);
	    gl.end();

      // if-statement alternates colours.
      if (i % 2 == 0){
      	// crimson.
      	gl.color(220./256.,20./256.,50./256.);
      } else{
      	// firebrick.
        gl.color(178./256.,34./256.,34./256.);
      }
      // Draw bars.
	    gl.begin(gl.QUADS);
	      gl.vertex(x1, 0.);
	      gl.vertex(x1, y);
	      gl.vertex(x2, y2);
	      gl.vertex(x2, 0.);
	    gl.end();

      // Draw year markers at the beginning of each January.
      if (i % 12 == 0){
        gl.begin(gl.LINES);
        	gl.color(.3,.3,.3);
        	gl.vertex(x1, 0.);
		      gl.vertex(x1, 1.);
		    gl.end();
		  }
	  }

	  //gl.popMatrix();
	}
};

int main(int argc, char *argv[]){
	MyApp().start();
}
