#!/usr/bin/env ruby

require 'json'
require 'net/http'
require 'sequel'

# Returns the first search result and returns Spotify artist uri.
def get_artist_uri(artist)
  url = "http://ws.spotify.com/search/1/artist.json?q=\"#{artist}\""
  resp = Net::HTTP.get_response(URI.parse(URI.escape(url)))
  data = resp.body

  # Converts returned JSON to hash.
  result = JSON.parse(data)

  # Access the uri of the first result.
  return result["artists"][0]["href"]
end

# Returns an array of albums by the passed artist.
def list_albums(artist)
  albums = Array.new

  base_url = "http://ws.spotify.com/lookup/1/.json"
  uri = get_artist_uri(artist)
  url = "#{base_url}?uri=#{uri}&extras=album"
  resp = Net::HTTP.get_response(URI.parse(url))
  data = resp.body

  # Converts returned JSON to hash.
  result = JSON.parse(data)

  # If the hash has 'Error' as a key, raise an error.
  if result.has_key? 'Error'
    raise "web service error"
  end

  # Put together array of artist's albums.
  for album in result["artist"]["albums"]
    # Scrub '#'.
    # Replace '/' with a space.
    puts album["album"]["name"].gsub('#','').gsub('/',' ')

    # Add to albums array.
    albums << album["album"]["name"]
  end

  return albums
end

# Reports the number of times that 'subject' was checked out per month.
# Subject can be a string or array.
def query(subject)
  #DB = Sequel.connect("mysql2://mat259:V1sual1zat1on@tango.mat.ucsb.edu/spl1", :max_connections => 10)
  DB = Sequel.connect("mysql2://mat259:mat259@127.0.0.1/spl1", :max_connections => 10)

  DB.from(:title).join(:activity, :bib => :bib).join(:kind, :item => :item).where(:title.like(*subject), :kind => /cd$/, :o.extract(:year) => 2006..2012 ).group_and_count(:o.extract(:year), :o.extract(:month)).all
end

# Retrieves artist's albums and queries the array. 
def artist_checkout(artist)
  return query(list_albums(artist))
end

# Queries the provided array.
def album_checkout(*album_list)
  return query(album_list)
end

# Prints data to a JSON file and on-screen in an easy to copy string.
def output(artist)
  temp = album_checkout(artist)

  File.open("result.json","w") do |f|
    f.write( JSON.pretty_generate(JSON.parse(temp.to_json)))
  end

  easy = "int data[#{7*12}] = {"
  temp.each do |row|
    easy << row[:count].to_s
    if row[:"extract(year FROM `o`)"] == 2012 && row[:"extract(month FROM `o`)"] == 12
      easy << ".};"
    else
      easy << "., "
    end
  end

  puts easy
end